﻿#Example usage 
#This script requires PowerShell 7.2.00 or higher
#.\DeployFilesOnline.ps1 -SiteUrl "https://your-domain.sharepoint.com/sites/pwa" -IsOnline
#.\DeployFilesOnline.ps1 -SiteUrl "https://thorappsprojdevpaid.sharepoint.com/sites/devtest_copy" -IsOnline

#This script should now work with SharePoint Online, SharePoint Server 2016, and SharePoint Server 2019. Remember to install the appropriate PnP module for your target environment:
# Install SharePoint PnP PowerShell module if not already installed
# Install-Module -Name PnP.PowerShell -Force


#Add-Type -Path "<path to your Microsoft.SharePoint.Client.dll>"
#Add-Type -Path "<path to your Microsoft.SharePoint.Client.Runtime.dll>"
#Add-Type -Path "<path to your Microsoft.SharePoint.Client.Publishing.dll>"
#Add-Type -Path "<path to your Microsoft.SharePoint.Client.WebPartPages.dll>"

#Add-Type -Path "C:\Program Files\Common Files\microsoft shared\Web Server Extensions\16\ISAPI\Microsoft.SharePoint.Client.dll"
#Add-Type -Path "C:\Program Files\Common Files\microsoft shared\Web Server Extensions\16\ISAPI\Microsoft.SharePoint.Client.Runtime.dll"

Param(
    [string]$SiteUrl,
    [switch]$IsOnline
)

# File paths relative to the script location
$Files = @(
    "Index.html",
    "LicenseAgreement.aspx",
    "EntitySelection.aspx",
    "Images/AppLogo.png",
    "Styles/styles.css",
    "Scripts/EntitySelection.js",
    "Scripts/InitPage.js",
    "Scripts/IOBaseEntity.js",
    "Scripts/IOLookupTables.js",
    "Scripts/IOLookupEntries.js",
    "Scripts/IOCustomFields.js",
    "Scripts/IOProjectDetailPages.js",
    "Scripts/IOEnterpriseProjectTypes.js",
    "Scripts/IOPWAViews.js",
    "Scripts/IOResources.js"
    "Scripts/IOSiteColumns.js",
    "Scripts/IOContentTypes.js"
)

$LibraryName = "ThorAppsPCM"
$PWASitePath = $SiteUrl -replace 'https?://[^/]+'


function Add-WebPartToClassicPage($pageUrl, $webPartXml, $zoneId, $zoneIndex) {

#    Write-Host "PageUrl: $pageUrl"
    # Get the context
    $ctx = Get-PnPContext
    # Get the Limited Web Part Manager
    $file = $ctx.Web.GetFileByServerRelativeUrl($pageUrl) #$webPartPageUrl)
    $limitedWebPartManager = $file.GetLimitedWebPartManager([Microsoft.SharePoint.Client.WebParts.PersonalizationScope]::Shared)
    $ctx.Load($limitedWebPartManager)
    $ctx.ExecuteQuery()

    # Check if the web part already exists on the page
    $existingWebParts = $limitedWebPartManager.WebParts
    $ctx.Load($existingWebParts)
    $ctx.ExecuteQuery()

    $webPartsToRemove = $existingWebParts # | Where-Object { $_.WebPart.Title -eq "PWA Configuration Manager" }

    Write-Host "Removing existing web parts with title 'PWA Configuration Manager'..."
    foreach ($webPart in $webPartsToRemove) {
        Remove-PnPWebPart -ServerRelativePageUrl $pageUrl -Identity $webPart.Id
    }
    $ctx.ExecuteQuery()
    Write-Host "Existing web parts removed successfully."

    # Add the Content Editor Web Part
    $webPartImportResult = $limitedWebPartManager.ImportWebPart($webPartXml)
    $newWebPart = $limitedWebPartManager.AddWebPart($webPartImportResult.WebPart, $zoneId, $zoneIndex)
    $ctx.Load($newWebPart)
    $ctx.ExecuteQuery()
    
}

# Check if a PnP connection already exists
if (-not $PnPConnection) {
    if ($IsOnline) {
        # If we are connecting to SharePoint Online, create a new connection with MFA
        Connect-PnPOnline -Url $SiteUrl -Interactive
    }
    else {
        # If we are connecting to SharePoint Server, create a new connection using Windows authentication
        #Connect-PnPOnline -Url $SiteUrl -UseWebLogin
        Connect-PnPOnline -Url $SiteUrl -TransformationOnPrem -CurrentCredential
    }
}

# Check if the library exists, create it if it does not
$library = Get-PnPList -Identity $LibraryName -ErrorAction SilentlyContinue
if (!$library) {
    $out = New-PnPList -Title $LibraryName -Template DocumentLibrary -EnableVersioning
    $library = Get-PnPList -Identity $LibraryName
}

# Add the necessary content types to the library
$webPartPageContentType = Get-PnPContentType -Identity "Web Part Page"
$documentContentType = Get-PnPContentType -Identity "Document"

if ($webPartPageContentType) {
    $out = Add-PnPContentTypeToList -List $library -ContentType $webPartPageContentType
    #Set-PnPDefaultContentTypeToList -List $library -ContentType $webPartPageContentType
}

Add-PnPContentTypeToList -List $library -ContentType $documentContentType

# Upload files to the document library
$sourceFile = ""
foreach ($file in $Files) {
    $folder = [System.IO.Path]::GetDirectoryName($file)
    $libraryFolder = if ($folder -eq '.') { $LibraryName } else { "$LibraryName/$folder" }
    $sourcFile = $LibraryName + "\" + $file
    if ($minorVersioningEnabled) {
        Add-PnPFile -Path $sourcFile -Folder $libraryFolder -Publish
    }
    else {
        Add-PnPFile -Path $sourcFile -Folder $libraryFolder
    }
}

#$pageContentType = Get-PnPContentType -Identity "Web Part Page"
$templateFilePath = "$LibraryName\TemplateWebPartPage.aspx"

# Add the web part pages
$pageUrl = "$PWASitePath/$LibraryName/PWAGovernance.aspx"
#if ((Get-PnPFile -Url $pageUrl -ErrorAction SilentlyContinue) -ne $null) {
#    Write-Host "The page $pageUrl already exists. Skipping upload and configuration."
#} else {
if ($minorVersioningEnabled) {
    Add-PnPFile -Path $templateFilePath -Folder $LibraryName -NewFileName "PWAGovernance.aspx" -Publish
}
else
{
    Add-PnPFile -Path $templateFilePath -Folder $LibraryName -NewFileName "PWAGovernance.aspx"
}

$webPartXml = @"
<?xml version="1.0" encoding="utf-8"?>
<WebPart xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.microsoft.com/WebPart/v2">
  <Title>PWA Configuration Manager</Title>
  <FrameType>None</FrameType>
  <Description>Web Part for managing PWA configuration settings.</Description>
  <IsIncluded>true</IsIncluded>
  <ZoneID>Header</ZoneID>
  <PartOrder>0</PartOrder>
  <FrameState>Normal</FrameState>
  <Height />
  <Width />
  <AllowRemove>true</AllowRemove>
  <AllowZoneChange>true</AllowZoneChange>
  <AllowMinimize>true</AllowMinimize>
  <AllowConnect>true</AllowConnect>
  <AllowEdit>true</AllowEdit>
  <AllowHide>true</AllowHide>
  <IsVisible>true</IsVisible>
  <DetailLink />
  <HelpLink />
  <HelpMode>Modeless</HelpMode>
  <Dir>Default</Dir>
  <PartImageSmall />
  <MissingAssembly>Cannot import this Web Part.</MissingAssembly>
  <PartImageLarge>/_layouts/15/images/mscontl.gif</PartImageLarge>
  <IsIncludedFilter />
  <Assembly>Microsoft.SharePoint, Version=16.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c</Assembly>
  <TypeName>Microsoft.SharePoint.WebPartPages.ContentEditorWebPart</TypeName>
  <ContentLink xmlns="http://schemas.microsoft.com/WebPart/v2/ContentEditor">$PWASitePath/$LibraryName/Index.html</ContentLink>
  <Content xmlns="http://schemas.microsoft.com/WebPart/v2/ContentEditor" />
  <PartStorage xmlns="http://schemas.microsoft.com/WebPart/v2/ContentEditor" />
</WebPart>
"@

Add-WebPartToClassicPage -pageUrl $pageUrl -webPartXml $webPartXml -zoneId "Header" -zoneIndex 0
Write-Host "The page $pageUrl has been uploaded and configured with the Content Editor Web Part."
#}

# Disconnect from SharePoint Online or SharePoint Server
Disconnect-PnPOnline
Get-Date -Format "dddd MM/dd/yyyy HH:mm K"