// Ensure the global SP and PS variables are defined
/// <reference path="PS.debug.js" />
/// <reference path="SP.debug.js" />
/// <reference path="SP.Runtime.js" />
var SP = SP || {};
var PS = PS || {};

class IOBaseEntity {
    constructor(apiUrl, fileInput, logDiv, progressBar) {
        this.apiUrl = apiUrl;
        this.fileInput = fileInput;
        this.logDiv = logDiv;
        this.progressBar = progressBar;
        this.logText = '';
        this.logRows = 0;
    }

    get_name() {
        throw new Error("Not Implemented!");
    }

    async request(method, endpoint, body) {
        const digest = await this.getDigest();
        let contentType = '';
        if (method == 'POST') {
            contentType = 'application/json;odata=verbose';
        }
        else {
            //contentType = 'application/json';
            contentType = 'application/json;odata=verbose';
        }
        const options = {
            method: method,
            headers: {
                //"Accept": "application/json", //;odata=verbose",
                "Accept": "application/json;odata=verbose",
                'Content-Type': contentType,
                'X-RequestDigest': digest
            }
        };

        if (body) {
            options.body = JSON.stringify(body);
        }
        const response = await fetch(`${this.apiUrl}${endpoint}`, options);

        if (!response.ok) {
            const errorText = await response.text();
            console.error("Error:", errorText);
            throw new Error(`HTTP error! status: ${response.status}`);
        }
        return response.json();
    }

    isSharePointOnline() {
        var isSPO = false;
        var url = window.location.href.toLowerCase();
        //if (url.indexOf('-my.sharepoint.com') !== -1 || url.indexOf('-admin.sharepoint.com') !== -1) {
        if (url.indexOf('.sharepoint.com') !== -1) {
            isSPO = true;
        }
        return isSPO;
    }

    async getDigest() {
        const requestDigest = document.getElementById('__REQUESTDIGEST').value;
        return requestDigest;
    }

    async export() { throw new Error("Not Implemented!"); }
    async import() { throw new Error("Not Implemented!"); }

    updateProgress(percentage) {
        if (percentage <= 0) {
            percentage = 0;
            this.progressBar.style.backgroundColor = "transparent";
            this.progressBar.style.color = "black";
            this.progressBar.style.width = "100%";
        }
        else {
            this.progressBar.style.backgroundColor = "#4CAF50";
            this.progressBar.style.color = "white";
            this.progressBar.style.width = `${percentage}px`;
        }

        this.progressBar.textContent = `${percentage}%`;
    }


    //Doesn't seem to be working
    syncSetTimeout(timeoutPeriod) {
        return new Promise((resolve, reject) => {
            this.appendLog(`Waiting ${timeoutPeriod} milliseconds for PWA changes to commit`);
            setTimeout(() => {
                resolve();
            }, timeoutPeriod);
            resolve();
        });
    }

    executeQuery(context, onSuccess, onError) {
        return new Promise((resolve, reject) => {
            context.executeQueryAsync(
                () => {
                    try {
                        onSuccess();
                        resolve();
                    }
                    catch (error) {
                        reject(error);
                    }
                },
                (sender, args) => {
                    try {
                        onError(sender, args);
                        resolve();
                    }
                    catch (error) {
                        reject(error);
                    }
                    //reject(args.get_message());
                }
            );
        });
    }

    count(sharePointItemCollection) {
        let enumerator = sharePointItemCollection.getEnumerator();
        let count = 0;
        while (enumerator.moveNext()) {
            count++;
        }
        return count;
    }

    openFile() {
        const fileInput = this.fileInput;

        if (!fileInput) {
            throw new Error('File input element not found');
        }
        if (!fileInput.files) {
            throw new Error('No file selected');
        }
        const file = fileInput.files[0];
        if (!file) {
            throw new Error('No file selected');
        }
        return file;
    }

    async readFileAsText(file) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.onload = (event) => {
                const text = event.target.result;
                resolve(text);
            };
            reader.onerror = (event) => {
                reject(event.target.error);
            };
            reader.readAsText(file);
        });
    }

    appendLog(message) {
        if (this.logText != '') {
            this.logText = this.logText.concat("<br/>");
        }
        this.logText = this.logText.concat(message);
        this.logRows += 1;
        console.log(message);
        this.logDiv.innerHTML = this.logText;
        //alert(this.logText);
    }

    appendWarning(message) {
        this.appendLog(`<span style="color: orange">${message}</span>`);
    }

    appendError(message) {
        this.appendLog(`<span style="color: red">${message}</span>`);
    }

    appendAndThrowError(message) {
        this.appendError(message);
        throw new Error(message);
    }

    clearLog() {
        this.logText = '';
        this.logRows = 0;
        this.logDiv.innerHTML = "There are no log entries to display";
    }

    startNewLog(message) {
        this.clearLog();
        this.appendLog("Starting new log");
        if (message) {
            if (message != '') {
                this.appendLog(message);
            }
        }
    }
}