// Ensure the global SP and PS variables are defined
/// <reference path="PS.debug.js" />
/// <reference path="SP.debug.js" />
/// <reference path="SP.Runtime.js" />
var SP = SP || {};
var PS = PS || {};

class IOEnterpriseProjectTypes extends IOBaseEntity {
    constructor(apiUrl, fileInput, logTextArea, progressBar) {
        super(apiUrl, fileInput, logTextArea, progressBar);
    }

    get_name() {
        return "Enterprise Project Types";
    }

    async export() {
        this.startNewLog("Retrieving Enterprise Project Types");
        //Using REST
        const data = await this.request('GET', '/_api/ProjectServer/EnterpriseProjectTypes');
        //const enterpriseProjectTypes = data.value;
        const enterpriseProjectTypes = data.d.results;
        var count = enterpriseProjectTypes.length;
        var index = 0;
        var progress = 0;

        for (let ept of enterpriseProjectTypes) {
            // Assuming that the NewProjectPageName can be accessed from the EPT object
            const newProjectPageName = ept.NewProjectPageName;

            // Assuming that ProjectDetailPages is a collection related to the EPT that can be fetched with a separate request
            const pdpResponse = await this.request('GET', `/_api/ProjectServer/EnterpriseProjectTypes('${ept.Id}')/projectDetailPages`);
            //const projectDetailPages = pdpResponse.value;
            const projectDetailPages = pdpResponse.d.results;

            // Add these properties to the EPT object
            ept.NewProjectPageName = newProjectPageName;
            ept.ProjectDetailPages = projectDetailPages;

            index++;
            progress = Math.round((index / count) * 100);
            this.updateProgress(progress);

        }

        this.appendLog(`Found ${enterpriseProjectTypes.length} Enterprise Project Types in PWA`);
        var pwaConfig = {
            EnterpriseProjectTypes: enterpriseProjectTypes
        };

        const blob = new Blob([JSON.stringify(pwaConfig, null, 4)], { type: "application/json" });
        const url = URL.createObjectURL(blob);
        const link = document.createElement('a');
        link.href = url;
        link.download = 'EnterpriseProjectTypes.json';
        link.click();
    }

    async import() {
        //return new Promise(async (resolve, reject) => {
            //try {
                this.startNewLog('');

                const file = this.openFile();
                this.appendLog("Reading Enterprise Project Types from file");
                const fileContent = await this.readFileAsText(file);

                const pwaConfig = JSON.parse(fileContent);
                const enterpriseProjectTypes = pwaConfig.EnterpriseProjectTypes;
                if (enterpriseProjectTypes == null) {
                    reject(`No '${this.get_name()}' were found in the selected file.`);
                }

                var count = enterpriseProjectTypes.length;
                this.appendLog(`Found ${count} Enterprise Project Types in file`);
                let projectContext = PS.ProjectContext.get_current();
                this.appendLog("Reading Enterprise Project Types from PWA");
                var existingEPTs = projectContext.get_enterpriseProjectTypes();
                projectContext.load(existingEPTs, 'Include(Name, Id)');

                this.appendLog("Reading Project Details Pages from PWA");
                var existingPDPs = projectContext.get_projectDetailPages();

                projectContext.load(existingPDPs, 'Include(Name, Id)');

                await this.executeQuery(projectContext, () => {

                    this.appendLog(`Found ${existingEPTs.get_count()} Enterprise Project Types in PWA`);
                },
                (sender, args) => {
                    throw new Error(`FAILED to read Enterprise Project Types from PWA'. Error: '${args.get_message()} : ${args.toString()}'.`);
                });

                var existingEPT = null;
                var index = 0;
                var progress = 0;
                for (const ept of enterpriseProjectTypes) {
                    if (!(ept.Selected == null || ept.Selected == true)) {
                        this.appendLog(`Enterprise Project Type '${ept.Name}' is not selected for Import.`);
                    }
                    else {
                        existingEPT = null;
                        const eptEnumerator = existingEPTs.getEnumerator();
                        while (eptEnumerator.moveNext()) {
                            let existing = eptEnumerator.get_current();

                            if (existing.get_name() == ept.Name) {
                                existingEPT = existing;

                                break;
                            }
                        }

                        try {
                            var updateCreateMessage = 'create';
                            if (existingEPT != null) {
                                updateCreateMessage = 'update';
                                this.appendLog(`Enterprise Project Type '${ept.Name}' already exists. Updating it.`);

                                this.setEPT(existingEPT, ept, existingPDPs, false);
                            } else {
                                this.appendLog(`Enterprise Project Type '${ept.Name}' does not exist, Creating it.`);
                                var newEPT = new PS.EnterpriseProjectTypeCreationInformation();

                                //const newGuid = SP.Guid.newGuid().toString();
                                //this.appendLog('New Id is :' + newGuid);
                                const newGuid = ept.Id;
                                newEPT.set_id(newGuid);

                                this.setEPT(newEPT, ept, existingPDPs, true);

                                existingEPTs.add(newEPT);
                            }

                            existingEPTs.update();
                            //projectContext.load(existingEPTs);
                            await this.executeQuery(projectContext, () => {
                                this.appendLog(`Enterprise Project Type '${ept.Name}' has been ${updateCreateMessage}d.`);
                            },
                                (sender, args) => {
                                    //this.appendError(`FAILED to create Enterprise Project Type '${newEPT.get_name()}'. Error: '${args.get_message()}'.`);
                                    throw new Error(`FAILED to ${updateCreateMessage} Enterprise Project Type '${ept.Name}'. Error: '${args.get_message()}'.`);
                                }
                            );

                        } catch (error) {
                            //Log error and move on to next resource
                            this.appendError(`Error importing Enterprise Project Type '${ept.Name}': ${error.toString()}`);
                        }
                    }

                    index++;
                    progress = Math.round((index / count) * 100);
                    this.updateProgress(progress);
                }
                //resolve();
            //}
            //catch (error) {
            //    reject(error);
            //}
        //});
    }

    setEPT(pwaEPT, fileEPT, existingPDPs, isNew) {
        try {
            pwaEPT.set_name(fileEPT.Name);
            pwaEPT.set_description(fileEPT.Description);
            pwaEPT.set_isDefault(fileEPT.IsDefault);
            pwaEPT.set_order(fileEPT.Order);
            pwaEPT.set_projectPlanTemplateId(fileEPT.ProjectPlanTemplateId);
            pwaEPT.set_workflowAssociationId(fileEPT.WorkflowAssociationId);
            pwaEPT.set_workspaceTemplateName(fileEPT.WorkspaceTemplateName);

            if (isNew) { //Can not set associated PDP's on an Existing EPT, try deleting and re-creating the EPT instead
                var associatedPDPs = [];
                var position = 0;
                var associatedPDP = null;
                var existingPDP = null;
                var hasCreatePDP = false;
                for (const eptPDP of fileEPT.ProjectDetailPages) {

                    associatedPDP = new PS.ProjectDetailPageCreationInformation();
                    existingPDP = this.selectExistingPDPByName(existingPDPs, eptPDP.Name);

                    associatedPDP.set_id(existingPDP.get_id()); //Trying this, but it doesn't work either

                    if ((eptPDP.PageType == 2) && (!hasCreatePDP)) {
                        associatedPDP.set_isCreate(true);
                        hasCreatePDP = true;
                    }
                    else {
                        associatedPDP.set_isCreate(false);
                    }
                    associatedPDP.set_position(position);
                    associatedPDPs.push(associatedPDP);
                    position++;
                }

                pwaEPT.set_projectDetailPages(associatedPDPs);
            }
            else {
                this.appendWarning(`WARNING: Can not update associated PDP's on an Existing EPT '${fileEPT.Name}', you'll need to delete the EPT and re-import it instead. Other changes will be applied to the existing EPT.`);
            }
        }
        catch (Error) {
            //console.error(error);
            throw new Error(error);
        }
    }

    selectExistingPDPByName(existingPDPs, pdpName) {
        const pdpEnumerator = existingPDPs.getEnumerator();
        while (pdpEnumerator.moveNext()) {
            let existing = pdpEnumerator.get_current();

            if (existing.get_name() == pdpName) {
                return existing;
            }
        }
        return null;
    }
}