// Ensure the global SP and PS variables are defined
/// <reference path="PS.debug.js" />
/// <reference path="SP.debug.js" />
/// <reference path="SP.Runtime.js" />
var SP = SP || {};
var PS = PS || {};

class IOLookupTables extends IOBaseEntity {
    constructor(apiUrl, fileInput, logTextArea, progressBar) {
        super(apiUrl, fileInput, logTextArea, progressBar);
    }

    get_name() {
        return "Lookup Tables";
    }

    async export() {
        this.startNewLog("Retrieving Lookup Tables");
        //Using REST
        const data = await this.request('GET', '/_api/ProjectServer/LookupTables');
        //const lookupTables = data.value;
        const lookupTables = data.d.results;

        this.appendLog(`Found ${lookupTables.length} Lookup Tables in PWA`);
        var pwaConfig = {
            LookupTables: lookupTables
        };

        const blob = new Blob([JSON.stringify(pwaConfig, null, 4)], { type: "application/json" });
        const url = URL.createObjectURL(blob);
        const link = document.createElement('a');
        link.href = url;
        link.download = 'LookupTables.json';
        link.click();
    }

    async import() {
        return new Promise(async (resolve, reject) => {
            try {
                this.startNewLog('');

                const file = this.openFile();
                this.appendLog("Reading Lookup Tables from file");
                const fileContent = await this.readFileAsText(file);
                const pwaConfig = JSON.parse(fileContent);
                const lookupTables = pwaConfig.LookupTables;
                if (lookupTables == null) {
                    reject(`No '${this.get_name()}' were found in the selected file.`);
                }

                var count = lookupTables.length;
                this.appendLog(`Found ${count} Lookup Tables in file`);
                let projectContext = PS.ProjectContext.get_current();
                this.appendLog("Reading Lookup Tables from PWA");
                var existingTables = projectContext.get_lookupTables();
                projectContext.load(existingTables, 'Include(Name, Id)');
                await this.executeQuery(projectContext, async () => {

                    this.appendLog(`Found ${existingTables.get_count()} Lookup Tables in PWA`);

                    var existingTable = null;
                    var index = 0;
                    var progress = 0;
                    for (const table of lookupTables) {
                        if (!(table.Selected == null || table.Selected == true)) {
                            this.appendLog(`Lookup Table '${table.Name}' is not selected for Import.`);
                        }
                        else {
                            existingTable = null;
                            const tableEnumerator = existingTables.getEnumerator();
                            while (tableEnumerator.moveNext()) {
                                let existing = tableEnumerator.get_current();

                                if (existing.get_name() == table.Name) {
                                    existingTable = existing;

                                    break;
                                }
                            }

                            if (existingTable != null) {
                                this.appendLog(`Lookup Table '${table.Name}' already exists. Skipping.`);
                            } else {
                                //try {
                                this.appendLog(`Lookup Table '${table.Name}' does not exist, Creating it.`);
                                const newGuid = SP.Guid.newGuid().toString();
                                this.appendLog('New Id is :' + newGuid);

                                var newLookupTable = new PS.LookupTableCreationInformation();
                                newLookupTable.set_id(newGuid);
                                //newLookupTable.set_id(table.Id); //Must use a new Id :(
                                newLookupTable.set_name(table.Name);
                                newLookupTable.set_sortOrder(table.SortOrder);
                                let newMasks = [];
                                //PS.LookupTableMaskSequence
                                for (const mask of table.Masks.results) {
                                    let newMask = new PS.LookupMask();
                                    newMask.set_length(mask.Length);
                                    newMask.set_maskType(mask.MaskType);
                                    newMask.set_separator(mask.Separator);
                                    newMasks.push(newMask);
                                }
                                newLookupTable.set_masks(newMasks);

                                //New Entry to TEST
                                var entries = [];
                                newLookupTable.set_entries(entries); //Must supply an Entries object or we get a NullReference Exception ont the server, which is returns to the client as 'Unknown error'

                                //appendLog(`NewLookupTable JSON='${JSON.stringify(newLookupTable)}'`);
                                existingTables.add(newLookupTable);
                                existingTables.update();
                                projectContext.load(existingTables);
                                await this.executeQuery(projectContext, () => {
                                    //projectContext.executeQueryAsync((sender, args) => {
                                    this.appendLog(`Lookup Table '${newLookupTable.get_name()}' has been created.`);
                                },
                                    (sender, args) => {
                                        throw new Error(`FAILED to create Lookup Table '${newLookupTable.get_name()}'. Error: '${args.get_message()}'.`);
                                    }
                                );
                                //} catch (error) {
                                //    throw new Error(`Error creating lookup table '${table.Name}': ${error}`);
                                //}
                            }
                        }

                        index++;
                        progress = Math.round((index / count) * 100);
                        this.updateProgress(progress);
                    }
                    resolve();
                },
                    //(response) => { reject(response) }
                    (sender, args) => {
                        reject(`FAILED to read Lookup tables from PWA'. Error: '${args.get_message()}'.`);
                    }
                ); //Failed
            }
            catch
            {
                reject(error);
            }
        });
    }


}