// Ensure the global SP and PS variables are defined
/// <reference path="PS.debug.js" />
/// <reference path="SP.debug.js" />
/// <reference path="SP.Runtime.js" />
var SP = SP || {};
var PS = PS || {};

class IOProjectDetailPages extends IOBaseEntity {
    constructor(apiUrl, fileInput, logTextArea, progressBar) {
        super(apiUrl, fileInput, logTextArea, progressBar);
    }

    get_name() {
        return "Project Details Pages";
    }

    async export() {
        this.startNewLog("Retrieving Project Detail Pages");
        //Using REST
        const data = await this.request('GET', `/_api/web/lists/getbytitle('Project Detail Pages')/items?$select=FileLeafRef,Id,File/ServerRelativeUrl,ContentType/Id,ContentType/Name,TitlePDP,PageType,Description&$expand=File,ContentType`);
        //const pDPs = data.value;
        const pDPs = data.d.results;

        var count = pDPs.length;

        //if a custom field has a lookup table and a default value, we need to extract that and include it in the exported data
        var index = 0;
        var progress = 0;
        let projectContext = PS.ProjectContext.get_current();
        var webpartDetails = null;
        for (const pdp of pDPs) {

            //Download the actual page aspx file, this contains the pages web part zone layouts, so we need it to later upload.
            pdp.FileContent = await this.getFileContent(pdp.File.ServerRelativeUrl);

        //Add item level detail to PDPs here
        //console.log("Getting web part details");
        //webpartDetails = await this.getPageWebPartProperties(projectContext, `/Project Detail Pages/${pdp.FileLeafRef}`);
        //console.log(`WebPartDetails = '${JSON.stringify(webpartDetails)}'`);
        //pdp.WebParts = webpartDetails;

            index++;
            progress = Math.round((index / count) * 100);
            this.updateProgress(progress);
        }

        this.appendLog(`Found ${count} Project Detail Pages in PWA`);

        var pwaConfig = {
            ProjectDetailPages: pDPs
        };

        const blob = new Blob([JSON.stringify(pwaConfig, null, 4)], { type: "application/json" });
        const url = URL.createObjectURL(blob);
        const link = document.createElement('a');
        link.href = url;
        link.download = 'ProjectDetailPages.json';
        link.click();
    }

    async import() {
        return new Promise(async (resolve, reject) => {
            try {
                this.startNewLog('');

                const file = this.openFile();
                this.appendLog("Reading Project Detail Pages from file");
                const fileContent = await this.readFileAsText(file);
                const pwaConfig = JSON.parse(fileContent);
                const projectDetailPages = pwaConfig.ProjectDetailPages;
                if (projectDetailPages == null) {
                    reject(`No '${this.get_name()}' were found in the selected file.`);
                }

                var count = projectDetailPages.length;
                var progress = 0;
                var index = 0;
                this.appendLog(`Found ${count} Project Detail Pages in file`);
                let projectContext = PS.ProjectContext.get_current();

                var web = projectContext.get_web();
                projectContext.load(web, 'ServerRelativeUrl');
                var list = web.get_lists().getByTitle('Project Detail Pages');
                projectContext.load(list);

                await this.executeQuery(projectContext, () => {
                    /*SUCCESS*/
                },
                    (sender, args) => {
                        //this.appendError(`FAILED to load existing Project Details Pages List from PWA. Error: '${args.get_message()}'.`);
                        reject(`FAILED to load existing Project Details Pages List from PWA. Error: '${args.get_message()}'.`);
                    }
                );

                var listItems = list.getItems(SP.CamlQuery.createAllItemsQuery());
                projectContext.load(listItems, 'Include(FileLeafRef)');

                await this.executeQuery(projectContext, () => {
                    /*SUCCESS*/
                },
                    (sender, args) => {
                        //this.appendError(`FAILED to load existing Project Details Pages from PWA. Error: '${args.get_message()}'.`);
                        reject(`FAILED to load existing Project Details Pages from PWA. Error: '${args.get_message()}'.`);
                    }
                );

                var existingFiles = listItems.get_data().map(item => item.get_item('FileLeafRef'));

                for (let i = 0; i < projectDetailPages.length; i++) {
                    let pdp = projectDetailPages[i];

                    if (!(pdp.Selected == null || pdp.Selected == true)) {
                        this.appendLog(`Project Details Page '${pdp.Name}' is not selected for Import.`);
                    }
                    else {
                        if (existingFiles.includes(pdp.FileLeafRef)) {
                            this.appendLog(`Page ${pdp.FileLeafRef} already exists. Skipping...`);
                            continue;
                        }

                        //var newPageUrl = `${web.get_serverRelativeUrl()}/Project Detail Pages/${pdp.FileLeafRef}`;
                        //var newFile = list.get_rootFolder().get_files().addTemplateFile(newPageUrl, SP.TemplateFileType.standardPage);
                        //var newItem = newFile.get_listItemAllFields();

                        //newItem.set_item('TitlePDP', pdp.TitlePDP);
                        ////newItem.set_item('ContentType', pdp.ContentType.Name);
                        //console.log(`pdp.ContentType.Id.StringValue = ${pdp.ContentType.Id.StringValue}`);
                        //newItem.set_item('ContentTypeId', pdp.ContentType.Id.StringValue);
                        //newItem.set_item('PageType', pdp.PageType);
                        //newItem.set_item('Description', pdp.Description);
                        //newItem.update();

                        var newPageUrl = `${web.get_serverRelativeUrl()}/Project Detail Pages/${pdp.FileLeafRef}`;
                        var createInfo = new SP.FileCreationInformation();
                        createInfo.set_url(newPageUrl);
                        createInfo.set_content(new SP.Base64EncodedByteArray());
                        for (var j = 0; j < pdp.FileContent.length; j++) {
                            createInfo.get_content().append(pdp.FileContent.charCodeAt(j));
                        }
                        var newFile = list.get_rootFolder().get_files().add(createInfo);
                        var newItem = newFile.get_listItemAllFields();

                        newItem.set_item('TitlePDP', pdp.TitlePDP);
                        //newItem.set_item('ContentType', pdp.ContentType.Name);
                        console.log(`pdp.ContentType.Id.StringValue = ${pdp.ContentType.Id.StringValue}`);
                        newItem.set_item('ContentTypeId', pdp.ContentType.Id.StringValue);
                        newItem.set_item('PageType', pdp.PageType);
                        newItem.set_item('Description', pdp.Description);
                        newItem.update();

                        projectContext.load(newItem);

                        await this.executeQuery(projectContext, () => {
                            this.appendLog(`Imported Project Detail Page '${pdp.FileLeafRef}'!`);
                        },
                            (sender, args) => {
                                reject(`FAILED to import Project Details Page '${pdp.FileLeafRef}' into PWA. Error: '${args.get_message()}'.`);
                            }
                        );
                    }

                    index++;
                    progress = Math.round((index / count) * 100);
                    this.updateProgress(progress);

                }

                resolve();
            }
            catch (error) {
                reject(error);
            }
        });
    }

    async getFileContent(fileUrl) {

        const response = await fetch(`${this.apiUrl}/_api/web/GetFileByServerRelativeUrl('${fileUrl}')/$value`, {
            headers: {
                "accept": "text/plain;odata=verbose",
                "content-type": "text/plain;odata=verbose"
            }
        });

        if (!response.ok) {
            throw new Error(`HTTP error! status: ${response.status}`);
        }

        const fileContent = await response.text();
        return fileContent;
    }


    //async getPageWebPartProperties(projectContext, pageUrl) {
    //    try {
    //        const fileName = pageUrl;
    //        const web = projectContext.get_web();
    //        projectContext.load(web);
    //        await this.executeQuery(projectContext, () => {/*Success*/ }, (sender, args) => { throw new Error(`FAILED to load web for PWA'. Error: '${args.get_message()}'.`); });
    //        pageUrl = `${web.get_serverRelativeUrl()}${pageUrl}`;

    //        const webPartPage = web.getFileByServerRelativeUrl(pageUrl);
    //        const webPartManager = webPartPage.getLimitedWebPartManager(SP.WebParts.PersonalizationScope.shared);
    //        const webParts = webPartManager.get_webParts();

    //        projectContext.load(webParts);
    //        await this.executeQuery(projectContext, () => {/*Success*/ }, (sender, args) => { throw new Error(`FAILED to read web parts from ${pageUrl}'. Error: '${args.get_message()}'.`); });

    //        var webparts = [];

    //        const webPartCount = webParts.get_count();
    //        var sortIndex = 0;
    //        for (let i = 0; i < webPartCount; i++) {
    //            let webPartInfo = {};
    //            const webPartDef = webParts.get_item(i);
    //            const webPart = webPartDef.get_webPart();

    //            projectContext.load(webPartDef);
    //            projectContext.load(webPart);
    //            await this.executeQuery(projectContext, () => {/*Success*/ }, (sender, args) => { throw new Error(`FAILED to load web part on Page ${pageUrl}'. Error: '${args.get_message()}'.`); });

    //            const props = webPart.get_properties();
    //            projectContext.load(props);
    //            await this.executeQuery(projectContext, () => {/*Success*/ }, (sender, args) => { throw new Error(`FAILED to get web part properties on Page ${pageUrl}'. Error: '${args.get_message()}'.`); });

    //            //console.log(`${fileName} Web Part ${i + 1}:`, props.get_fieldValues());
    //            var fieldValues = props.get_fieldValues();
    //            //const xmlConfig = buildWebPartConfigXml(fieldValues);

    //            webPartInfo.Id = webPartDef.get_id().toString();
    //            webPartInfo.Title = webPart.get_title();
    //            webPartInfo.Hidden = webPart.get_hidden();
    //            webPartInfo.IsClosed = webPart.get_isClosed();
    //            webPartInfo.SubTitle = webPart.get_subtitle();
    //            webPartInfo.ZoneIndex = webPart.get_zoneIndex();
    //            webPartInfo.SortIndex = sortIndex;

    //            webPartInfo.Properties = fieldValues;
    //            //if (!this.isSharePointOnline()) {
    //            webPartInfo.XMLConfiguration = await this.getWebPartXmlOnPremises(pageUrl, webPartInfo.Id); //webPartDef.get_id());
    //            //}
    //            //else {
    //            //    webPartInfo.XMLConfiguration = await this.getWebPartConfigXml(`${pageUrl}`, webPartInfo.Id);
    //            //}
    //            //webPartInfo.XMLConfiguration = await this.getWebPartConfigXml(`${this.apiUrl}${fileName}`, webPartInfo.Id);
    //            //webPartInfo.XMLConfiguration = await this.getWebPartConfigXml(`${pageUrl}`, webPartInfo.Id);
    //            webparts.push(webPartInfo);

    //            sortIndex++;
    //        }
    //        return webparts;
    //    } catch (error) {
    //        console.error("Error:", error);
    //    }
    //}

    ////Stole this from PnP and chatGPT converted it.
    //async getWebPartXmlOnPremises(serverRelativePageUrl, webPartId) {
    //    const hostUrl = window.location.protocol + "//" + window.location.host;
    //    const webUrl = _spPageContextInfo.webServerRelativeUrl;
    //    const pageUrl = hostUrl + serverRelativePageUrl;
    //    console.log(`pageUrl encoded=${encodeURIComponent(pageUrl)}`);
    //    console.log(`pageUrl=${pageUrl}`);
    //    console.log(`webPartId=${webPartId}`);

    //    //const exportUrl = `${webUrl}/_vti_bin/exportwp.aspx?pageurl=${encodeURIComponent(pageUrl)}&guidstring=${webPartId}`;
    //    const exportUrl = `${webUrl}/_vti_bin/exportwp.aspx?pageurl=${pageUrl}&guidstring=${webPartId}`;

    //    const response = await fetch(exportUrl, {
    //        method: 'GET',
    //        credentials: 'include',
    //        headers: {
    //            'User-Agent': 'Mozilla/5.0 (Windows NT; Windows NT 6.2; de-DE) pnprocks/5.1.19041.1'
    //        }
    //    });

    //    if (response.ok) {
    //        const webPartXml = await response.text();
    //        return webPartXml;
    //    } else {
    //        throw new Error(`HTTP error! status: ${response.status}`);
    //    }
    //}

    ////The SOAP method is returning http 500, we're assuming it has been deprecated
    //async getWebPartConfigXml(pageUrl, webPartId) {
    //    return new Promise((resolve, reject) => {
    //        const webServiceUrl = `${this.apiUrl}/_vti_bin/WebPartPages.asmx`;
    //        console.log(`webServiceUrl = ${webServiceUrl}`);
    //        console.log(`pageUrl = ${pageUrl}`);
    //        const soapEnvelope =
    //            `<?xml version="1.0" encoding="utf-8"?>
    //    <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
    //      <soap:Body>
    //        <GetWebPart2 xmlns="http://microsoft.com/sharepoint/webpartpages">
    //          <pageUrl>${pageUrl}</pageUrl>
    //          <storageKey>${webPartId}</storageKey>
    //          <behavior>Version3</behavior>
    //        </GetWebPart2>
    //      </soap:Body>
    //    </soap:Envelope>`;

    //        const xhr = new XMLHttpRequest();
    //        xhr.open('POST', webServiceUrl, true);
    //        xhr.setRequestHeader('Content-Type', 'text/xml; charset=utf-8');
    //        xhr.setRequestHeader('SOAPAction', 'http://microsoft.com/sharepoint/webpartpages/GetWebPart2');
    //        xhr.onreadystatechange = function () {
    //            if (xhr.readyState === 4) {
    //                if (xhr.status === 200) {
    //                    const xml = xhr.responseXML;
    //                    resolve(xml);
    //                } else {
    //                    console.error(`Response Text: ${xhr.responseText}`);
    //                    console.error(`Ready State: ${xhr.readyState}`);
    //                    console.error(`Status: ${xhr.status}`);
    //                    console.error(`Status Text: ${xhr.statusText}`);
    //                    reject(new Error(`Error fetching web part config XML: ${xhr.status} (${xhr.statusText})`));
    //                }
    //            }
    //        };
    //        xhr.send(soapEnvelope);
    //    });
    //}
}