﻿
function loadScript(url) {
    return new Promise(resolve => {
        var script = document.createElement("script");
        script.type = "text/javascript";

        if (script.readyState) { // For IE
            script.onreadystatechange = function () {
                if (script.readyState === "loaded" || script.readyState === "complete") {
                    script.onreadystatechange = null;
                    resolve(`Loaded script '${url}'`);
                }
            };
        } else { // For other browsers
            script.onload = function () {
                resolve(`Loaded script '${url}'`);
            };
        }

        const apiUrl = _spPageContextInfo.webAbsoluteUrl;
        script.src = apiUrl + url;
        document.getElementsByTagName("head")[0].appendChild(script);
    });
}

function loadStyleSheet(url) {
    return new Promise(resolve => {
        var link = document.createElement("link");
        link.type = "text/css";

        if (link.readyState) { // For IE
            link.onreadystatechange = function () {
                if (link.readyState === "loaded" || link.readyState === "complete") {
                    link.onreadystatechange = null;
                    resolve(`Loaded stylesheet '${url}'`);
                }
            };
        } else { // For other browsers
            link.onload = function () {
                resolve(`Loaded stylesheet '${url}'`);
            };
        }

        const apiUrl = _spPageContextInfo.webAbsoluteUrl;
        link.href = apiUrl + url;
        link.rel = "stylesheet";
        document.getElementsByTagName("head")[0].appendChild(link);
    });
}

function openLicenseAgreement() {
    var options = {
        url: _spPageContextInfo.siteAbsoluteUrl + '/ThorAppsPCM/LicenseAgreement.aspx',
        title: 'ThorApps PWA Configuration Manager (PCM) License Agreement - \u00A9 ThorApps 2023',
        allowMaximize: false,
        showClose: true,
        width: 800,
        height: 600,
    };
    SP.SOD.execute('sp.ui.dialog.js', 'SP.UI.ModalDialog.showModalDialog', options);
}

function agreeToLicenseTerms() {
    var isChecked = document.getElementById('iAgreeToLicenseTerms').checked;
    if (isChecked) {
        setButtonsAndInputsState(false);
    }
    else {
        setButtonsAndInputsState(true);
    }
}

function openSelection(entityName) {
    var options = {
        url: _spPageContextInfo.siteAbsoluteUrl + `/ThorAppsPCM/EntitySelection.aspx?entityName=${entityName}`,
        title: 'ThorApps PWA Configuration Manager (PCM) - \u00A9 ThorApps 2023',
        allowMaximize: false,
        showClose: true,
        width: 590,
        height: 600,
    };
    SP.SOD.execute('sp.ui.dialog.js', 'SP.UI.ModalDialog.showModalDialog', options);
}
async function loadDependencies() {
    await loadStyleSheet('/ThorAppsPCM/Styles/Styles.css');
    //<link rel="stylesheet" href="<%= SP.Utilities.Utility.getLayoutsPageUrl('/ThorAppsPCM/Styles/Styles.css') %>" type="text/css">
    let scriptVersion = "0.5"; //apend script version to end of the script load requests to ensure the latest version is loaded

    //await loadScript("/_layouts/15/init.js");
    //await loadScript("/_layouts/15/sp.runtime.js");
    await loadScript("/_layouts/15/sp.js");
    await loadScript("/_layouts/15/ps.js");
    await loadScript(`/ThorAppsPCM/Scripts/IOBaseEntity.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOLookupTables.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOLookupEntries.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOCustomFields.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOProjectDetailPages.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOEnterpriseProjectTypes.js?v=${scriptVersion}`);
    //await loadScript(`/ThorAppsPCM/Scripts/IOPWAViews.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOResources.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOSiteColumns.js?v=${scriptVersion}`);
    await loadScript(`/ThorAppsPCM/Scripts/IOContentTypes.js?v=${scriptVersion}`);
}

function setButtonsAndInputsState(isDisabled) {

    const allButtons = document.querySelectorAll("button");
    const exportButtons = Array.from(allButtons).filter(button => button.textContent.trim() === 'Export');
    const selectButtons = Array.from(allButtons).filter(button => button.textContent.trim() === 'Selection');
    const importButtons = Array.from(allButtons).filter(button => button.textContent.trim() === 'Import');
    const fileInputs = document.querySelectorAll("input[type='file']");
    const elements = [...exportButtons, ...selectButtons, ...fileInputs];

    elements.forEach(element => element.disabled = isDisabled);
    
    document.getElementById('iAgreeToLicenseTerms').disabled = isDisabled;

    var isIAgreeChecked = document.getElementById('iAgreeToLicenseTerms').checked;

    for (const importButton of importButtons) {
        if (isDisabled) {
            importButton.disabled = isDisabled;
        }
        else {
            
            if (isIAgreeChecked) {
                let fileImport = importButton.parentNode.nextElementSibling.firstChild;
                if (fileImport.files && fileImport.files.length > 0) {
                    importButton.disabled = isDisabled; //same as false
                }
                else {
                    importButton.disabled = true;
                }
            }
            else {
                importButton.disabled = !isIAgreeChecked
            }
        }
    }
}

function initPCMEntity(entityProcessor, titleSpanId, exportButtonId, importButtonId, importFileId, progressBarId) {

    const titleSpan = document.getElementById(titleSpanId);
    const exportButton = document.getElementById(exportButtonId);
    const importButton = document.getElementById(importButtonId);
    const importFile = document.getElementById(importFileId);
    const progressBar = document.getElementById(progressBarId);

    entityProcessor.updateProgress(0);

    exportButton.addEventListener('click', async () => {
        setButtonsAndInputsState(true);
        entityName = entityProcessor.get_name();
        titleSpan.textContent = entityName;
        entityProcessor.updateProgress(0);
        try {
            entityProcessor.appendLog(`Starting ${entityName} export`);

            await entityProcessor.export();
            entityProcessor.updateProgress(100);
            entityProcessor.appendLog("Export finished successfully");
        } catch (error) {
            entityProcessor.appendError(`<span style="color: Red">Error exporting ${entityName}: ${error} \nstack: ${error.stack}</span>`);
            entityProcessor.updateProgress(0);
            entityProcessor.appendError("Export exitted on error");
        }
        setButtonsAndInputsState(false);
    });

    importButton.addEventListener('click', async () => {
        setButtonsAndInputsState(true);
        entityName = entityProcessor.get_name();
        titleSpan.textContent = entityName;
        entityProcessor.updateProgress(0);
        if (confirm(`Are you sure you want to import ${entityName}?`)) {
            try {
                entityProcessor.appendLog(`Starting ${entityName} import`);
                await entityProcessor.import();
                entityProcessor.updateProgress(100);
                entityProcessor.appendLog("Import finished");
            } catch (error) {
                entityProcessor.appendError(`<span style="color: Red">Error importing ${entityName}: ${error} \nstack: ${error.stack}</span>`)
                //entityProcessor.updateProgress(0); //leave it where it is
                entityProcessor.appendError("Import exitted on error");
            }
        }
        setButtonsAndInputsState(false);
    });

    importFile.addEventListener('change', async () => {
        setButtonsAndInputsState(false);
    //    // Check if a file has been selected
    //    if (this.files && this.files.length > 0) {
    //        // If a file has been selected, enable the import button
    //        setButtonsAndInputsState(false);
    //    } else {
    //        // If no file has been selected, disable the import button
    //        importButton.disabled = true;
    //    }
    });
}

async function initPCMPage() {
    await loadDependencies();

    setButtonsAndInputsState(true);
    document.getElementById('openLicenseTerms').href = 'javascript: openLicenseAgreement();';
    var iAgreeToLicenseTerms = document.getElementById('iAgreeToLicenseTerms');
    iAgreeToLicenseTerms.onchange = agreeToLicenseTerms;

    const apiUrl = _spPageContextInfo.webAbsoluteUrl;
    var activityLog = document.getElementById('activityLog');
    const lookupTables = new IOLookupTables(apiUrl, document.getElementById('fileLookupTablesInput'), activityLog, document.getElementById('progressBarLookupTables'));
    const lookupEntries = new IOLookupEntries(apiUrl, document.getElementById('fileLookupEntriesInput'), activityLog, document.getElementById('progressBarLookupEntries'));
    const customFields = new IOCustomFields(apiUrl, document.getElementById('fileCustomFieldsInput'), activityLog, document.getElementById('progressBarCustomFields'));
    const PDPs = new IOProjectDetailPages(apiUrl, document.getElementById('filePDPsInput'), activityLog, document.getElementById('progressBarPDPs'));
    const EPTs = new IOEnterpriseProjectTypes(apiUrl, document.getElementById('fileEPTsInput'), activityLog, document.getElementById('progressBarEPTs'));
    //const pwaViews = new IOPWAViews(apiUrl, document.getElementById('filePWAViewsInput'), activityLog, document.getElementById('progressBarPWAViews'));
    const resources = new IOResources(apiUrl, document.getElementById('fileResourcesInput'), activityLog, document.getElementById('progressBarResources'));
    const siteColumns = new IOSiteColumns(apiUrl, document.getElementById('fileSiteColumnsInput'), activityLog, document.getElementById('progressBarSiteColumns'));
    const contentTypes = new IOContentTypes(apiUrl, document.getElementById('fileContentTypesInput'), activityLog, document.getElementById('progressBarContentTypes'));

    initPCMEntity(lookupTables, 'titleLookupTablesSpan', 'exportLookupTablesButton', 'importLookupTablesButton', 'fileLookupTablesInput', 'progressBarLookupTables');
    initPCMEntity(lookupEntries, 'titleLookupEntriesSpan', 'exportLookupEntriesButton', 'importLookupEntriesButton', 'fileLookupEntriesInput', 'progressBarLookupEntries');
    initPCMEntity(customFields, 'titleCustomFieldsSpan', 'exportCustomFieldsButton', 'importCustomFieldsButton', 'fileCustomFieldsInput', 'progressBarCustomFields');
    initPCMEntity(PDPs, 'titlePDPsSpan', 'exportPDPsButton', 'importPDPsButton', 'filePDPsInput', 'progressBarPDPs');
    initPCMEntity(EPTs, 'titleEPTsSpan', 'exportEPTsButton', 'importEPTsButton', 'fileEPTsInput', 'progressBarEPTs');
    //initPCMEntity(pwaViews, 'titlePWAViewsSpan', 'exportPWAViewsButton', 'importPWAViewsButton', 'filePWAViewsInput', 'progressBarPWAViews');
    initPCMEntity(resources, 'titleResourcesSpan', 'exportResourcesButton', 'importResourcesButton', 'fileResourcesInput', 'progressBarResources');
    initPCMEntity(siteColumns, 'titleSiteColumnsSpan', 'exportSiteColumnsButton', 'importSiteColumnsButton', 'fileSiteColumnsInput', 'progressBarSiteColumns');
    initPCMEntity(contentTypes, 'titleContentTypesSpan', 'exportContentTypesButton', 'importContentTypesButton', 'fileContentTypesInput', 'progressBarContentTypes');

    //activityLog.innerHTML = "Show me this text";
    lookupTables.clearLog();
    iAgreeToLicenseTerms.disabled = false;

}

SP.SOD.executeOrDelayUntilScriptLoaded(initPCMPage, 'sp.js');
